# -- coding: utf-8 --
"""
Created on Wed Apr  8 17:53:04 2020

@author: Klaus Reinold
"""


from graphics_and_games_klassen import Rechteck, Figur, Text, Ereignisbehandlung, Kreis, Dreieck
from random import randint


class Monster(Figur):
    """
    Verwaltet das Monster
    """


    def __init__(self, x, y, rahmen):
        """
        Legt die Figur und die Mundstellung fest.
        """    
        super().__init__()
        self.rahmen = rahmen
        self.PositionSetzen(x, y)
        self.GroesseSetzen(self.rahmen.basisGroesse)
        self.anzahl = 0
        self.mundIstZu = True
        self.Schliessen("gelb")
        self.WinkelSetzen(0)      
        self.pausiert = False
        self.leben = 3
        self.text = Text()
        self.text.TextSetzen(str(self.anzahl)+" Punkte; "+str(self.leben)+" Leben")
    

    def GeschlossenGelb(self):
        """
        Stellt das gelb gefärbte Monster mit geschlossenem Mund dar.
        """    
        self.EigeneFigurLoeschen()
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, "gelb")
        self.FigurteilFestlegenRechteck(0, -2, 50, 4, "schwarz")
        self.FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz")
    


    def GeschlossenOrange(self):
        """
        Stellt das orange gefärbte Monster mit geschlossenem Mund dar.
        """    
        self.EigeneFigurLoeschen()
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, "orange")
        self.FigurteilFestlegenRechteck(0, -2, 50, 4, "schwarz")
        self.FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz")
    


    def GeschlossenRot(self):
        """
        Stellt das rot gefärbte Monster mit geschlossenem Mund dar.
        """   
        self.EigeneFigurLoeschen()
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, "rot")
        self.FigurteilFestlegenRechteck(0, -2, 50, 4, "schwarz")
        self.FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz")
    


    def GeschlossenGruen(self):
        """
        Stellt das grün gefärbte Monster mit geschlossenem Mund dar.
        """    
        self.EigeneFigurLoeschen()
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, "grün")
        self.FigurteilFestlegenRechteck(0, -2, 50, 4, "schwarz")
        self.FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz")
    


    def Offen(self, farbe):
        """
        Stellt das Monster in der angegebenen Farbe mit offenem Mund dar.
        -- Parameter farbe die gewünschte Farbe
        """    
        self.EigeneFigurLoeschen()
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, farbe)
        self.FigurteilFestlegenDreieck(0, 0, 50, -10, 50, 10, "schwarz")
        self.FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz")
    
    

    def Schliessen(self, farbe):
        """
        Stellt das in einer der Farben gelb, orange, rot bzw. grün mit geschlossenem Mund dar.
        -- Parameter farbe die gewünschte Farbe
        """    
        if farbe == "gelb": 
            self.GeschlossenGelb()
        if farbe == "orange":
            self.GeschlossenOrange()
        if farbe == "rot":
            self.GeschlossenRot()
        if farbe == "grün":
            self.GeschlossenGruen()


    def Oeffnen(self, farbe):
        """
        Stellt das in einer der Farben gelb, orange, rot bzw. grün mit offenem Mund dar.
        -- Parameter farbe die gewünschte Farbe
        """    
        if farbe == "gelb" or farbe == "orange" or farbe == "rot" or farbe == "grün":
            self.Offen(farbe)
    
    
    def PositionSetzen(self, x, y):
        self.xPos = x
        self.yPos = y
        super().PositionSetzen(x * self.rahmen.basisGroesse + self.rahmen.basisGroesse / 2, y * self.rahmen.basisGroesse + self.rahmen.basisGroesse / 2)

    def Reagieren(self, monster):
        """
        Das Spielelement kann auf des Monster reagieren
        -- Parameter monster das Monster
        """  
        pass
    
        
    def XPositionGeben(self):
        """
        Gibt die xPos zurück
        """
        return self.xPos
    
    def YPositionGeben(self):
        """
        Gibt die yPos zurück
        """
        return self.yPos
        


    def TasteGedrueckt (self,taste):
        """
        Die Aktionsmethode für gedrückte Tasten.
        -- Parameter taste KeyCode der gedrückten Taste
        """    
        if taste == "left":
            self.Drehen(90)
        elif taste == "up":
            self.Gehen(10)
        elif taste == "right":
            self.Drehen(-90)
        elif taste == "down":
            self.Drehen(180)
        elif taste == " ":
            self.Gehen(50)  
        self.pausiert = False              
             

    def AktionAusfuehren(self):
        """
        Das Monster geht maximal bis zum Rand
        """
        if self.mundIstZu:
            self.Oeffnen("gelb")
            self.mundIstZu = False
        else:
        
            self.Schliessen("gelb")
            self.mundIstZu = True
        if not self.pausiert:
            if self.WinkelGeben() == 0:
                if self.xPos + 1 < 31 and not self.rahmen.zellen[self.xPos + 1][self.yPos]==None:
                    self.PositionSetzen(self.xPos + 1, self.yPos)   
                else:
                    self.pausiert = True
                    if self.xPos > 30:
                        Verloren()
            elif self.WinkelGeben() == 90:
                if self.yPos > 0 and not self.rahmen.zellen[self.xPos][self.yPos-1]==None:
                    self.PositionSetzen(self.xPos, self.yPos-1)
                else:
                    self.pausiert = True
                    if self.yPos < 0:
                        Verloren()
            elif self.WinkelGeben() == 180:
                if self.xPos > 0 and not self.rahmen.zellen[self.xPos - 1][self.yPos]==None:
                    self.PositionSetzen(self.xPos - 1, self.yPos)   
                else:
                    self.pausiert = True
                    if self.xPos < 0:
                        Verloren()
            elif self.WinkelGeben() == 270:
                if self.yPos < 32 and not self.rahmen.zellen[self.xPos][self.yPos + 1]==None:
                    self.PositionSetzen(self.xPos, self.yPos+1)
                else:
                    self.pausiert = True
                    if self.yPos > 31:
                        Verloren()
            if self.rahmen.zellen[self.xPos][self.yPos].HatKruemel():
                self.rahmen.zellen[self.xPos][self.yPos].KruemelEntfernen()
                self.anzahl = self.anzahl + 1
            
        self.text.TextSetzen(str(self.anzahl)+" Punkte; "+str(self.leben)+" Leben")
        
        if self.BeruehrtFarbe("rot"):
          self.leben = self.leben-1
          self.rahmen.AufStartpositionSetzen("rot")          
        if self.BeruehrtFarbe("blau"):
          self.leben = self.leben-1
          self.rahmen.AufStartpositionSetzen("blau")   
        if self.BeruehrtFarbe("gruen"):
          self.leben = self.leben-1
          self.rahmen.AufStartpositionSetzen("gruen")  

        if self.leben <= 0:
            self.rahmen.Anhalten()

        
        if self.anzahl >= 40:
            Gewonnen()
            
            
            

class Spielelement(Rechteck):
    """
    Oberklasse für alle Spielelementse
    Regelt die Groesse und die Positionierung
    """


    def __init__(self, x, y, rahmen):
        """
           Der Konstruktor legt die Position fest
        -- Parameter x x-Koordinate des Spielelementes
        -- Parameter y y-Koordinate des Spielelementes
        -- Parameter rahmen das Rahmenobjekt des Spiels
        """  
        super().__init__()
        self.xPos = x
        self.yPos = y
        self.rahmen = rahmen

     
    def Zeichen(self):
        pass

    def Positionieren(self, x, y):
        self.xPos = x
        self.yPos = y
        self.Zeichnen()


    
        
    def XPositionGeben(self):
        """
        Gibt die xPos zurück
        """
        return self.xPos
    
    def YPositionGeben(self):
        """
        Gibt die yPos zurück
        """
        return self.yPos
        









class Jaeger(Spielelement):
    """
    Verfolgt das Monster konsequent.
    """


    def __init__(self, x, y, rahmen):
        """
        Der Konstruktor legt Position, Groesse und Farbe fest
        -- Parameter x x-Koordinate des Rechtecks
        -- Parameter y y-Koordinate des Rechtecks
        -- Parameter rahmen das Rahmenobjekt des Spiels
        """  
        super().__init__(x, y, rahmen)
        self.xStart = x
        self.yStart = y
        self.GroesseSetzen(self.rahmen.basisGroesse/4, self.rahmen.basisGroesse/4)
        self.FarbeSetzen("rot")
        self.dreieck = Dreieck()
        self.dreieck.FarbeSetzen("rot")
        self.dreieck.GroesseSetzen(self.rahmen.basisGroesse/2, self.rahmen.basisGroesse*3/4)
        self.Zeichnen()

  
    def Zeichnen(self):
        """
        Zeichnen der Figur
        """
        self.PositionSetzen((self.XPositionGeben()+7/16)*self.rahmen.basisGroesse, self.YPositionGeben()*self.rahmen.basisGroesse)
        self.dreieck.PositionSetzen((self.XPositionGeben()+1/2)*self.rahmen.basisGroesse, (self.YPositionGeben()+0.25)*self.rahmen.basisGroesse)
  
  

    def AufStartpositionSetzen(self):
        """
        Setzt den Jäger auf die Startposition zurück
        """  
        self.Positionieren(self.xStart, self.yStart)

        self.Zeichnen()
  
  

    def AbstandGeben (self, x, y):
        """
        Berechnet den Kästchenabstand der gegebenen Koordinaten zum Monster
        -- return Kästchenabstand
        """  
        return abs(x - self.rahmen.MonsterGeben().XPositionGeben()) + abs(y - self.rahmen.MonsterGeben().YPositionGeben())
  
  

    def Bewegen(self):
        """
        Der Jäger bewegt sich direkt auf das Monster zu
        """  
        minAbstand = 100000
        xOpt = self.XPositionGeben()
        yOpt = self.YPositionGeben()
        if self.rahmen.IstWegelement(self.XPositionGeben(), self.YPositionGeben() - 1):      
            xOpt = self.XPositionGeben()
            yOpt = self.YPositionGeben() - 1
            minAbstand = self.AbstandGeben(xOpt, yOpt)
        
        if (self.AbstandGeben(self.XPositionGeben(), self.YPositionGeben() + 1) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben(), self.YPositionGeben() + 1)):
          xOpt = self.XPositionGeben()
          yOpt = self.YPositionGeben() + 1
          minAbstand = self.AbstandGeben(xOpt, yOpt)
        
        if (self.AbstandGeben(self.XPositionGeben() + 1, self.YPositionGeben()) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben() + 1, self.YPositionGeben())):
        
          xOpt = self.XPositionGeben() + 1
          yOpt = self.YPositionGeben()
          minAbstand = self.AbstandGeben(xOpt, yOpt)
        
        if (self.AbstandGeben(self.XPositionGeben() - 1, self.YPositionGeben()) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben() - 1, self.YPositionGeben())):
        
          xOpt = self.XPositionGeben() - 1
          yOpt = self.YPositionGeben()
          minAbstand = self.AbstandGeben(xOpt, yOpt)
        
        self.Positionieren(xOpt, yOpt)
        self.Zeichnen()
  

    def AktionAusfuehren(self):
        """
        Das Monster geht
        """  
        self.Bewegen()

    
  





class Fallensteller(Spielelement):
    """
    Geht konsequent zu einem Punkt 4 Einheiten vor dem Monster.
    """


    def __init__(self, x, y, rahmen):
        """
        Der Konstruktor legt Position, Groesse und Farbe fest
        -- Parameter x x-Koordinate des Rechtecks
        -- Parameter y y-Koordinate des Rechtecks
        -- Parameter rahmen das Rahmenobjekt des Spiels
        """  
        super().__init__(x, y, rahmen)
        self.xStart = x
        self.yStart = y
        self.GroesseSetzen(self.rahmen.basisGroesse/4, self.rahmen.basisGroesse/4)
        self.FarbeSetzen("gruen")
        self.dreieck = Dreieck()
        self.dreieck.FarbeSetzen("gruen")
        self.dreieck.GroesseSetzen(self.rahmen.basisGroesse/2, self.rahmen.basisGroesse*3/4)
        self.Zeichnen()
  

    def Zeichnen(self):
        """
        Zeichnen der Figur
        """
        self.PositionSetzen((self.XPositionGeben()+7/16)*self.rahmen.basisGroesse, self.YPositionGeben()*self.rahmen.basisGroesse)
        self.dreieck.PositionSetzen((self.XPositionGeben()+1/2)*self.rahmen.basisGroesse, (self.YPositionGeben()+0.25)*self.rahmen.basisGroesse)
  
  

    def AufStartpositionSetzen(self):
        """
        Setzt den Jäger auf die Startposition zurück
        """  
        self.Positionieren(self.xStart, self.yStart)
        self.Zeichnen()
  
  

    def AbstandGeben (self, x, y, x2, y2):
        """
        Berechnet den Kästchenabstand der gegebenen Koordinatenpaare
        -- return Kästchenabstand
        """  
        return (abs(x - x2) + abs(y - y2)) 
  
  

    def Bewegen(self):
        """
        Der Jäger bewegt sich direkt auf das Monster zu
        """  
        minAbstand = 100000
        xZiel = self.rahmen.MonsterGeben().XPositionGeben()
        yZiel = self.rahmen.MonsterGeben().YPositionGeben()
        if self.rahmen.MonsterGeben().WinkelGeben()==0:
            xZiel = xZiel + 4 
        elif self.rahmen.MonsterGeben().WinkelGeben()==90:
            yZiel = yZiel - 4 
        elif self.rahmen.MonsterGeben().WinkelGeben()==180:
            xZiel = xZiel - 4 
        elif self.rahmen.MonsterGeben().WinkelGeben()==270:
            yZiel = yZiel + 4 
        
        xOpt = self.XPositionGeben()
        yOpt = self.YPositionGeben()
        if  self.rahmen.IstWegelement(self.XPositionGeben(), self.YPositionGeben() - 1):
            xOpt = self.XPositionGeben()
            yOpt = self.YPositionGeben() - 1
            minAbstand = self.AbstandGeben(xOpt, yOpt, xZiel, yZiel)
        if (self.AbstandGeben(self.XPositionGeben(), self.YPositionGeben() + 1, xZiel, yZiel) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben(), self.YPositionGeben() + 1)):
          xOpt = self.XPositionGeben()
          yOpt = self.YPositionGeben() + 1
          minAbstand = self.AbstandGeben(xOpt, yOpt, xZiel, yZiel)       
        if (self.AbstandGeben(self.XPositionGeben() + 1, self.YPositionGeben(), xZiel, yZiel) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben() + 1, self.YPositionGeben())): 
          xOpt = self.XPositionGeben() + 1
          yOpt = self.YPositionGeben()
          minAbstand = self.AbstandGeben(xOpt, yOpt, xZiel, yZiel)     
        if (self.AbstandGeben(self.XPositionGeben() - 1, self.YPositionGeben(), xZiel, yZiel) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben() - 1, self.YPositionGeben())):
          xOpt = self.XPositionGeben() - 1
          yOpt = self.YPositionGeben()
          minAbstand = self.AbstandGeben(xOpt, yOpt, xZiel, yZiel)
        
        self.Positionieren(xOpt, yOpt)
        self.Zeichnen()
  
  

    def AktionAusfuehren(self):
        """
        Das Monster geht
        """  
        self.Bewegen()

  

class Chaot(Spielelement):
    """
    Verfolgt das Monster bis zu einem Abstand von 10 und bewegt sich dann zufällig
    """


    def __init__(self, x, y, rahmen):
        """
        Der Konstruktor legt Position, Groesse und Farbe fest
        -- Parameter x x-Koordinate des Rechtecks
        -- Parameter y y-Koordinate des Rechtecks
        -- Parameter rahmen das Rahmenobjekt des Spiels
        """  
        super().__init__(x, y, rahmen)
        self.xStart = x
        self.yStart = y
        self.GroesseSetzen(self.rahmen.basisGroesse/4, self.rahmen.basisGroesse/4)
        self.FarbeSetzen("blau")
        self.dreieck = Dreieck()
        self.dreieck.FarbeSetzen("blau")
        self.dreieck.GroesseSetzen(self.rahmen.basisGroesse/2, self.rahmen.basisGroesse*3/4)
        self.Zeichnen()
  
  
    def Zeichnen(self):
        """
        Zeichnen der Figur
        """
        self.PositionSetzen((self.XPositionGeben()+7/16)*self.rahmen.basisGroesse, self.YPositionGeben()*self.rahmen.basisGroesse)
        self.dreieck.PositionSetzen((self.XPositionGeben()+1/2)*self.rahmen.basisGroesse, (self.YPositionGeben()+0.25)*self.rahmen.basisGroesse)
 
  

    def AufStartpositionSetzen(self):
        """
        Setzt den Jäger auf die Startposition zurück
        """  
        self.Positionieren(self.xStart, self.yStart)
        self.Zeichnen()
  
  

    def AbstandGeben (self, x, y):
        """
        Berechnet den Kästchenabstand der gegebenen Koordinatenpaare
        -- return Kästchenabstand
        """  
        return (abs(x - self.rahmen.MonsterGeben().XPositionGeben()) + abs(y - self.rahmen.MonsterGeben().YPositionGeben()))
  
  

    def Bewegen(self):
        """
        Der Chaot bewegt sich
        """  
        minAbstand = 100000
        xOpt = self.XPositionGeben()
        yOpt = self.YPositionGeben()
        if self.AbstandGeben(self.XPositionGeben(), self.YPositionGeben())<=10:
            while True:
                zz = randint(0, 3)
                if zz == 0:
                    xOpt = self.XPositionGeben() + 1
                    yOpt = self.YPositionGeben()
                elif zz == 1:
                        xOpt = self.XPositionGeben() -1
                        yOpt = self.YPositionGeben()
                elif zz == 2:
                        xOpt = self.XPositionGeben()
                        yOpt = self.YPositionGeben() + 1
                elif zz == 3:
                        xOpt = self.XPositionGeben()
                        yOpt = self.YPositionGeben() - 1
                if self.rahmen.IstWegelement(xOpt, yOpt):
                    break
        else:
            if self.rahmen.IstWegelement(self.XPositionGeben(), self.YPositionGeben() - 1):      
                xOpt = self.XPositionGeben()
                yOpt = self.YPositionGeben() - 1
                minAbstand = self.AbstandGeben(xOpt, yOpt)   
            if (self.AbstandGeben(self.XPositionGeben(), self.YPositionGeben() + 1) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben(), self.YPositionGeben() + 1)):
              xOpt = self.XPositionGeben()
              yOpt = self.YPositionGeben() + 1
              minAbstand = self.AbstandGeben(xOpt, yOpt)
            if (self.AbstandGeben(self.XPositionGeben() + 1, self.YPositionGeben()) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben() + 1, self.YPositionGeben())):
              xOpt = self.XPositionGeben() + 1
              yOpt = self.YPositionGeben()
              minAbstand = self.AbstandGeben(xOpt, yOpt)
            if (self.AbstandGeben(self.XPositionGeben() - 1, self.YPositionGeben()) < minAbstand) and ( self.rahmen.IstWegelement(self.XPositionGeben() - 1, self.YPositionGeben())):  
              xOpt = self.XPositionGeben() - 1
              yOpt = self.YPositionGeben()
              minAbstand = self.AbstandGeben(xOpt, yOpt)
        
        self.Positionieren(xOpt, yOpt)
        self.Zeichnen()
  
  

    def AktionAusfuehren(self):
        """
        Das Monster geht
        """  
        self.Bewegen()


    
        


class Wegelement(Spielelement):
    """
    Verwaltet einen Krümel
    """


    def __init__(self, x, y, rahmen):
        """
        Legt Position und Figur fest
        -- Parameter x x-Position des Krümels
        -- Parameter y y-Position des Krümels
        """    
        super().__init__(x, y, rahmen)
        self.GroesseSetzen(self.rahmen.basisGroesse, self.rahmen.basisGroesse)
        self.FarbeSetzen("weiss")
        self.hatKruemel = True
        self.kreis = Kreis()
        self.kreis.FarbeSetzen("orange")
        self.kreis.RadiusSetzen(3)
        self.Zeichnen()
        
    def Zeichnen(self):
        """
        Zeichnen der Figur
        """
        self.PositionSetzen(self.XPositionGeben()*self.rahmen.basisGroesse, self.YPositionGeben()*self.rahmen.basisGroesse)
        self.kreis.PositionSetzen((self.XPositionGeben()+0.5)*self.rahmen.basisGroesse, (self.YPositionGeben()+0.5)*self.rahmen.basisGroesse)
  
    
    def KruemelEntfernen(self):
        self.kreis.Entfernen()
        self.hatKruemel = False
        
        
    def HatKruemel(self):
        return self.hatKruemel

    




class Verloren(Text):
    """
    Stellt den Verlusttext dar
    """

    def __init__(self):  
        """
        Legt Position und Text fest
        """
        super().__init__()
        self.TextSetzen("Du hast verloren")
        self.PositionSetzen(700, 20)
    




class Gewonnen(Text):
    """
    Stellt den Gewinntext dar
    """


    def __init__(self):  
        """
        Legt Position und Text fest
        """   
        super().__init__()
        self.TextSetzen("Du hast das Spiel gewonnen")
        self.PositionSetzen(10, 20)







class MauerRechteck(Spielelement):
    """
    Beschreibt ein Rechteck der (grauen) Mauer
    """

       
    def __init__(self, x, y, rahmen):
        """
        Die Initialisierung legt Position, Größe und Farbe fest
        -- Parameter x x-Koordinate des Rechtecks
        -- Parameter y y-Koordinate des Rechtecks
        -- Parameter breite Breite des Rechtecks
        -- Parameter hoehe Höhe des Rechtecks
        """
        super().__init__(x, y, rahmen)
        self.FigurteilFestlegenRechteck(-50, -50, 100, 100, "grau")
    




class Rahmen(Ereignisbehandlung):
    """
    Schafft den Rahmen fuer das Spiel.
    """
    

    def __init__(self):  
        """
        Legt das Spiel an
        """   
        super().__init__()
        r = Rechteck()
        r.FarbeSetzen("grau")
        r.GroesseSetzen(540, 600)
        self.basisGroesse = 20
        self.zellen = [[None]*31 for i in range (32)]
        
        for i in range (0, 26):
            self.zellen[i+1][1]=Wegelement(i+1, 1, self)
        for  i in range(0, 7):
            self.zellen[1][i+2]=Wegelement(1, i+2, self)
            self.zellen[26][i+2]=Wegelement(26, i+2, self)
        for  i in range(0, 14):
            self.zellen[i+7][5]=Wegelement(i+7, 5, self)
        for  i in range(0, 3):
            self.zellen[10][i+2]=Wegelement(10, i+2, self)
            self.zellen[17][i+2]=Wegelement(17, i+2, self)
        for  i in range(0, 24):
            self.zellen[6][i+1]=Wegelement(6, i+1, self)
            self.zellen[21][i+1]=Wegelement(21, i+1, self)
        for  i in range(0, 4):
            self.zellen[i+2][8]=Wegelement(i+2, 8, self)
            self.zellen[i+22][8]=Wegelement(i+22, 8, self)
        for  i in range(0, 10):
            self.zellen[i+9][8]=Wegelement(i+9, 8, self)
        for  i in range(0, 5):
            self.zellen[i+1][20]=Wegelement(i+1, 20, self)
            self.zellen[i+7][20]=Wegelement(i+7, 20, self)
            self.zellen[i+16][20]=Wegelement(i+16, 20, self)
            self.zellen[i+22][20]=Wegelement(i+22, 20, self)
        for  i in range(0, 4):
            self.zellen[12][i+20]=Wegelement(12, i+20, self)
            self.zellen[15][i+20]=Wegelement(15, i+20, self)
        for  i in range(0, 20):
            self.zellen[i+4][26]=Wegelement(i+4, 26, self)
        for  i in range(0, 26):
            self.zellen[i+1][28]=Wegelement(i+1, 28, self)
        for  i in range(0, 6):
            self.zellen[1][i+23]=Wegelement(1, i+23, self)
            self.zellen[26][i+23]=Wegelement(26, i+23, self)
        for  i in range(0, 3):
            self.zellen[3][i+21]=Wegelement(3, i+21, self)
            self.zellen[24][i+21]=Wegelement(24, i+21, self)   
        for  i in range(0, 6):
            self.zellen[i][14]=Wegelement(i, 14, self)
            self.zellen[i+22][14]=Wegelement(22+i, 14, self)
        self.zellen[2][23]= Wegelement(2, 23, self)
        self.zellen[25][23]=Wegelement(25, 23, self)
        self.zellen[24][26]=Wegelement(24, 26, self)
        self.zellen[21][25]=Wegelement(21, 25, self)
        for  i in range(0, 2):
            self.zellen[4][i+27]=Wegelement(4, i+27, self)
            self.zellen[24][i+27]=Wegelement(24, i+27, self)  
        for  i in range(0, 3):
            self.zellen[9][i+23]=Wegelement(9, i+23, self)
            self.zellen[18][i+23]=Wegelement(18, i+23, self) 
        for  i in range(0, 15):
            self.zellen[i+7][11]=Wegelement(i+7, 11, self)        
            self.zellen[i+7][17]=Wegelement(i+7, 17, self)
        for  i in range(0, 2):
            self.zellen[9][i+9]=Wegelement(9, 9+i, self)        
            self.zellen[18][i+9]=Wegelement(18, 9+i, self)        
        for  i in range(0, 5):
            self.zellen[9][i+12]=Wegelement(9, 12+i, self)        
            self.zellen[18][12+i]=Wegelement(18, 12+i, self)      
        for  i in range(0, 2):
            self.zellen[12][i+18]=Wegelement(12, i+18, self)        
            self.zellen[15][i+18]=Wegelement(15, i+18, self)   
        self.zellen[7][23]=Wegelement(7, 23, self)
        self.zellen[8][23]=Wegelement(8, 23, self)
        self.zellen[29][23]=Wegelement(19, 23, self)
        self.zellen[20][23]=Wegelement(20, 23, self)
        self.zellen[13][23]=Wegelement(13, 23, self)
        self.zellen[14][23]=Wegelement(14, 23, self)
        self.zellen[6][25]=Wegelement(6, 25, self)
        self.punkte = 0
        
        
        
 

        self.monster = Monster(4, 1, self)
        self.jaeger = Jaeger(13, 11, self)
        self.fallensteller = Fallensteller(12, 11, self)
        self.chaot = Chaot(14, 11, self)
        
        self.TaktdauerSetzen(400)
        self.Starten()
    
    
    def AufStartpositionSetzen(self, farbe):
        """
        Setzt das Monster in der gegebenen Farbe auf die Startposition
        -- Parameter farbe: Farbe des Monsters
        """
        if farbe=="rot":
            self.jaeger.AufStartpositionSetzen()
        elif farbe=="gruen":
            self.fallensteller.AufStartpositionSetzen()
        else: 
            self.chaot.AufStartpositionSetzen()
    

        

    def IstWegelement(self, x, y):
        """
        Gibt aus, ob Mauer voraus
        -- Parameter x x-Position
        -- Parameter y y-Position
        -- return Wahrheitswert
        """
        return isinstance(self.zellen[int(x)][int(y)], Wegelement)



    def TasteGedrueckt(self, taste):
        """
        nichts
        """
        pass
    
    def MonsterGeben(self):
        """
        Gibt das Monster
        -- return Monster
        """
        return self.monster

    def AktionAusfuehren(self):  
        """
        meldet den Taktschlag weiter
        """    
        self.jaeger.AktionAusfuehren()
        self.fallensteller.AktionAusfuehren()
        self.chaot.AktionAusfuehren()
    
Rahmen()
