# -*- coding: utf-8 -*-
"""
Created on Fry Jan 1 12:33:57 2021

@author: Johannes Neumeyer, Florian Janus, Albert Wiedemann
"""

from graphics_and_games_klassen import Rechteck, Kreis, Dreieck, Text


class ZimmerVerwaltung:
    # Legt den Namen fest und bereitet die Komponentenliste vor
    def __init__(self, name):
        self.name = name
        self.komponenten = []

    # Fügt eine neue Komponente in die Komponentenliste ein
    def KomponenteEintragen(self, neueKomponente):
        self.komponenten.append(neueKomponente)

    # Zeigt die Zustände der Komponenten des Zimmers an
    def ZustaendeAnzeigen(self):
        print("Komponenten des Zimmers:", self.name)
        for k in self.komponenten:
            print("    ", k.NameGeben() + ":", k.ZustandGeben())

    # Schaltet alle Hausgeräte dauerhaft ein
    def GeraeteEinschalten(self):
        for k in self.komponenten:
            if isinstance(k, Hausgeraet):
                k.Einschalten()

    # Schaltet alle Hausgeräte dauerhaft aus
    def GeraeteAusschalten(self):
        for k in self.komponenten:
            if isinstance(k, Hausgeraet):
                k.Ausschalten()

    # Setzt alle Hausgeräte auf Automatikbetrieb
    def GeraeteAutomatikSetzen(self):
        for k in self.komponenten:
            if isinstance(k, Hausgeraet):
                k.AutomatikAktivieren()


class Smartkomponente:
    # Legt den Namen der Komponente fest
    def __init__(self, name):
        self.name = name
        self.beobachter = []

    # Meldet den Namen der Komponente zurück
    def NameGeben(self):
        return self.name

    # Meldet eine Zustandsänderung an alle Beobachter
    def ZustandsAenderungMelden(self):
        for b in self.beobachter:
            b.AenderungMelden(self.ZustandGeben())

    # Registriert einen Zustandsbeobachter
    def BeobachterRegistrieren(self, neuerBeobachter):
        self.beobachter.append(neuerBeobachter)
        self.ZustandsAenderungMelden()

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        pass


class Hausgeraet(Smartkomponente):
    # Schaltet die Automatik ein
    def __init__(self, name):
        Smartkomponente.__init__(self, name)
        self.automatik = True

    # Wird von einem Messgerät bei einem neuen Wert aufgerufen
    def WertaenderungMelden(self, wert, geraet):
        pass

    # Schaltet die Komponente dauerhaft ein
    def Einschalten(self):
        self.automatik = False

    # Schaltet die Komponente dauerhaft ein
    def Ausschalten(self):
        self.automatik = False

    # Schaltet die Komponente dauerhaft ein
    def AutomatikAktivieren(self):
        self.automatik = True


class Messgeraet(Smartkomponente):
    # Richtet as Beobachterfeld ein
    def __init__(self, name):
        Smartkomponente.__init__(self, name)
        self.beobachter = []

    # Registriert einen Beobachter
    def BeobachterRegistrieren(self, neuerBeobachter):
        self.beobachter.append(neuerBeobachter)

    # Meldet den neuen Wert an alle Beobachter
    def MesswertAenderungMelden(self, wert):
        for h in self.beobachter:
            h.WertAenderungMelden(wert, self)

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        pass

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        pass


class Windsensor:
    # Legt den Namen und den Beobachter fest
    def __init__(self, namensZusatz):
        self.name = "Windsensor" + namensZusatz
        self.beobachter = []
        self.aktuellerWert = 0

    # Meldet den Namen des Geräts zurück
    # return Name
    def NameGeben(self):
        return self.name

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        try:
            wert = int(eingabe)
            return 0 <= wert <= 12
        except:
            return False

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter.append(beobachter)

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        wert = int(eingabe)
        if self.aktuellerWert != wert:
            self.aktuellerWert = wert
            for b in self.beobachter:
                b.WertaenderungMeldenWindsensor(self.aktuellerWert)

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        return str(self.aktuellerWert) + " Bft"


class Uhr:
    # Legt den Namen und den Beobachter fest
    def __init__(self, namensZusatz):
        self.name = "Uhr" + namensZusatz
        self.stunden = 0
        self.minuten = 0
        self.sekunden = 0
        self.beobachter = []

    # Meldet den Namen des Geräts zurück
    # return Name
    def NameGeben(self):
        return self.name

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter.append(beobachter)

    # Zerlegt den Eingabestring in seine Zeitbestandteile und setzt ggf. die aktuelle Zeit
    def Zerlegen(self, eingabe, setzen):
        try:
            teile = eingabe.split(":")
            s = 0
            m = 0
            h = 0
            if teile.len() == 1:
                h = int(teile[0])
            elif teile.len() == 2:
                h = int(teile[0])
                m = int(teile[1])
            elif teile.len() == 3:
                h = int(teile[0])
                m = int(teile[1])
                s = int(teile[3])
            else:
                return False
            if (0 <= s < 60) and (0 <= m < 60) and (0 <= h < 24):
                if setzen:
                    self.stunden = h
                    self.minuten = m
                    self.sekunden = s
                return True
            else:
                return False
        except:
            return False

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EinfgabePruefen(self, eingabe):
        return self.Zerlegen(eingabe, False)

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        if self.Zerlegen(eingabe, True):
            for b in self.beobachter:
                b.WertaenderungMeldenUhr(self.stunden * 10000 + self.minuten*100 + self.sekunden)

    # Meldet den Zustand der Uhr
    def ZustandGeben(self):
        if self.minuten < 10:
            minutenString = "0" + str(self.minuten)
        else:
            minutenString = str(self.minuten)
        if self.sekunden < 10:
            sekundenString = "0" + str(self.sekunden)
        else:
            sekundenString = str(self.sekunden)
        return str(self.stunden) + ":" + minutenString + ":" + sekundenString + " Uhr"


class Thermometer(Messgeraet):
    # Legt den Namen und den Beobachter fest
    def __init__(self, namensZusatz):
        Messgeraet.__init__(self, "Thermometer" + namensZusatz)
        self.aktuellerWert = 0

    # Prüft, on doe Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        try:
            wert = int(eingabe)
            return -50 <= wert <= 100
        except:
            return False

    # Setzt  den neuen Sensorwert
    def WertSetzen(self, eingabe):
        wert = int(eingabe)
        if self.aktuellerWert != wert:
            self.aktuellerWert = wert
            self.MesswertAenderungMelden(self.aktuellerWert)
            self.ZustandsAenderungMelden()

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        return str(self.aktuellerWert) + " °C"


class Bewegungsmelder(Messgeraet):
    # Legt den Namen und den Beobachter fest.
    def __init__(self, namensZusatz):
        Messgeraet.__init__(self, "Bewegungsmelder" + namensZusatz)
        self.an = False
        self.erlaubt = {"an": True, "ein": True, "wahr": True,
                        "true": True, "aus": False, "falsch": False, "false": False}

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        e = eingabe.lower()
        if e in self.erlaubt:
            return True
        else:
            return False

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        e = eingabe.lower()
        self.an = self.erlaubt[e]
        self.MesswertAenderungMelden(self.an)
        self.ZustandsAenderungMelden()

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        if self.an:
            return "Bewegung"
        else:
            return "Ruhe"


class Sonnensensor(Messgeraet):
    # Legt den Namen und den Beobachter fest.
    def __init__(self, namensZusatz):
        Messgeraet.__init__(self, "Sonnensensor" + namensZusatz)
        self.aktuellerWert = 0

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        try:
            wert = int(eingabe)
            return 0 <= wert <= 100
        except:
            return False

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        wert = int(eingabe)
        if self.aktuellerWert != wert:
            self.aktuellerWert = wert
            self.MesswertAenderungMelden(self.aktuellerWert)
            self.ZustandsAenderungMelden()

    # Meldet den Zustand des Sensors

    def ZustandGeben(self):
        return str(self.aktuellerWert) + " %"


class Waage:
    # Legt den Namen und den Beobachter fest.
    def __init__(self, namensZusatz):
        self.name = "Waage " + namensZusatz
        self.aktuellerWert = 0
        self.beobachter = []

    # Meldet den Namen des Geräts zurück
    # return Name
    def NameGeben(self):
        return self.name

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter.append(beobachter)

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        try:
            wert = int(eingabe)
            return 0 <= wert <= 10000
        except:
            return False

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        wert = int(eingabe)
        if self.aktuellerWert != wert:
            self.aktuellerWert = wert
            for b in self.beobachter:
                b.WertAenderungMelden(self.aktuellerWert)

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        return str(self.aktuellerWert) + " g"


class LampenSymbol:
    # Baut das Symbol auf
    def __init__(self, x, y):
        self.kabel = Rechteck()
        self.kabel.PositionSetzen(x, y)
        self.kabel.GroesseSetzen(1, 30)
        self.kabel.FarbeSetzen("schwarz")
        self.birne = Kreis()
        self.birne.RadiusSetzen(10)
        self.birne.PositionSetzen(x, y + 55)
        self.birne.FarbeSetzen("weiss")
        self.schirm = Dreieck()
        self.schirm.GroesseSetzen(50, 25)
        self.schirm.PositionSetzen(x, y + 30)
        self.schirm.FarbeSetzen("grau")

    # Meldet den neuen Zustand an das Symbol
    def AenderungMelden(self, zustandNeu):
        if zustandNeu == "ein":
            self.birne.FarbeSetzen("gelb")
        else:
            self.birne.FarbeSetzen("weiss")


class RollladenSymbol:
    # Baut das Symbol auf
    def __init__(self, x, y):
        self.lamelle0 = Rechteck()
        self.lamelle0.FarbeSetzen("schwarz")
        self.lamelle0.GroesseSetzen(100, 10)
        self.lamelle0.PositionSetzen(x, y)

        self.lamelle1 = Rechteck()
        self.lamelle1.FarbeSetzen("schwarz")
        self.lamelle1.GroesseSetzen(100, 10)
        self.lamelle1.PositionSetzen(x, y + 11)

        self.lamelle2 = Rechteck()
        self.lamelle2.FarbeSetzen("schwarz")
        self.lamelle2.GroesseSetzen(100, 10)
        self.lamelle2.PositionSetzen(x, y + 22)

        self.lamelle3 = Rechteck()
        self.lamelle3.FarbeSetzen("schwarz")
        self.lamelle3.GroesseSetzen(100, 10)
        self.lamelle3.PositionSetzen(x, y + 33)

        self.lamelle4 = Rechteck()
        self.lamelle4.FarbeSetzen("schwarz")
        self.lamelle4.GroesseSetzen(100, 10)
        self.lamelle4.PositionSetzen(x, y + 44)

        self.lamelle5 = Rechteck()
        self.lamelle5.FarbeSetzen("schwarz")
        self.lamelle5.GroesseSetzen(100, 10)
        self.lamelle5.PositionSetzen(x, y + 55)

        self.lamelle6 = Rechteck()
        self.lamelle6.FarbeSetzen("schwarz")
        self.lamelle6.GroesseSetzen(100, 10)
        self.lamelle6.PositionSetzen(x, y + 66)

        self.lamelle7 = Rechteck()
        self.lamelle7.FarbeSetzen("schwarz")
        self.lamelle7.GroesseSetzen(100, 10)
        self.lamelle7.PositionSetzen(x, y+77)

        self.lamelle8 = Rechteck()
        self.lamelle8.FarbeSetzen("schwarz")
        self.lamelle8.GroesseSetzen(100, 10)
        self.lamelle8.PositionSetzen(x, y+88)

    # Meldet den neuen Zustand an das Symbol
    def AenderungMelden(self, zustandNeu):
        if zustandNeu.lower() == "offen":
            self.lamelle1.SichtbarkeitSetzen(False)
            self.lamelle2.SichtbarkeitSetzen(False)
            self.lamelle3.SichtbarkeitSetzen(False)
            self.lamelle4.SichtbarkeitSetzen(False)
            self.lamelle5.SichtbarkeitSetzen(False)
            self.lamelle6.SichtbarkeitSetzen(False)
            self.lamelle7.SichtbarkeitSetzen(False)
            self.lamelle8.SichtbarkeitSetzen(False)
        else:
            self.lamelle1.SichtbarkeitSetzen(True)
            self.lamelle2.SichtbarkeitSetzen(True)
            self.lamelle3.SichtbarkeitSetzen(True)
            self.lamelle4.SichtbarkeitSetzen(True)
            self.lamelle5.SichtbarkeitSetzen(True)
            self.lamelle6.SichtbarkeitSetzen(True)
            self.lamelle7.SichtbarkeitSetzen(True)
            self.lamelle8.SichtbarkeitSetzen(True)


class HeizkoerperSymbol:
    # Baut das Symbol auf
    def __init__(self, x, y):
        self.innenkreise = []
        self.innenrechtecke = []
        for index in range(0, 8):
            self.HeizkoerperSegmentAnPos(x + index * 10, y)

    def HeizkoerperSegmentAnPos(self, x, y):
        # Segment außen
        oben = Kreis()
        oben.RadiusSetzen(10)
        oben.PositionSetzen(x + 10, y + 10)
        oben.FarbeSetzen("schwarz")

        mitte = Rechteck()
        mitte.GroesseSetzen(21, 40)
        mitte.PositionSetzen(x, y + 10)
        mitte.FarbeSetzen("schwarz")

        unten = Kreis()
        unten.RadiusSetzen(10)
        unten.PositionSetzen(x + 10, y + 50)
        unten.FarbeSetzen("schwarz")

        # Segment innen
        obenInnen = Kreis()
        obenInnen.RadiusSetzen(8)
        obenInnen.PositionSetzen(x + 10, y + 10)
        obenInnen.FarbeSetzen("weiss")
        self.innenkreise.append(obenInnen)

        mitteInnen = Rechteck()
        mitteInnen.GroesseSetzen(17, 42)
        mitteInnen.PositionSetzen(x + 2, y + 9)
        mitteInnen.FarbeSetzen("weiss")
        self.innenrechtecke.append(mitteInnen)

        untenInnen = Kreis()
        untenInnen.RadiusSetzen(8)
        untenInnen.PositionSetzen(x+10, y + 50)
        untenInnen.FarbeSetzen("weiss")
        self.innenkreise.append(untenInnen)

    # Meldet den neuen Zustand an das Symbol
    def AenderungMelden(self, zustandNeu):
        if zustandNeu.lower() == "ein":
            for k in self.innenkreise:
                k.FarbeSetzen("rot")
            for r in self.innenrechtecke:
                r.FarbeSetzen("rot")
        else:
            for k in self.innenkreise:
                k.FarbeSetzen("weiss")
            for r in self.innenrechtecke:
                r.FarbeSetzen("weiss")


class KuehlschrankSymbol:
    # Darstellung eines Kuehlschranks
    def __init__(self, x, y):
        self.eckeLO = Kreis()
        self.eckeLO.RadiusSetzen(15)
        self.eckeLO.PositionSetzen(x + 15, y + 15)
        self.eckeLO.FarbeSetzen("schwarz")
        self.randLinks = Rechteck()
        self.randLinks.GroesseSetzen(30, 100)
        self.randLinks.PositionSetzen(x, y + 15)
        self.randLinks.FarbeSetzen("schwarz")
        self.eckeLU = Kreis()
        self.eckeLU.RadiusSetzen(15)
        self.eckeLU.PositionSetzen(x + 15, y + 115)
        self.eckeLU.FarbeSetzen("schwarz")
        self.randUnten = Rechteck()
        self.randUnten.GroesseSetzen(60, 30)
        self.randUnten.PositionSetzen(x+15, y + 100)
        self.randUnten.FarbeSetzen("schwarz")
        self.randOben = Rechteck()
        self.randOben.GroesseSetzen(60, 30)
        self.randOben.PositionSetzen(x + 15, y)
        self.randUnten.FarbeSetzen("schwarz")
        self.eckeRO = Kreis()
        self.eckeRO.RadiusSetzen(15)
        self.eckeRO.PositionSetzen(x + 75, y + 15)
        self.eckeRO.FarbeSetzen("schwarz")
        self.eckeRU = Kreis()
        self.eckeRU.RadiusSetzen(15)
        self.eckeRU.PositionSetzen(x + 75, y + 115)
        self.eckeRU.FarbeSetzen("schwarz")
        self.randRechts = Rechteck()
        self.randRechts.GroesseSetzen(30, 100)
        self.randRechts.PositionSetzen(x + 60, y + 15)
        self.randRechts.FarbeSetzen("schwarz")

        self.eckeLOinnen = Kreis()
        self.eckeLOinnen.RadiusSetzen(15)
        self.eckeLOinnen.PositionSetzen(x + 17, y + 17)
        self.eckeLOinnen.FarbeSetzen("cyan")
        self.randLinksInnen = Rechteck()
        self.randLinksInnen.GroesseSetzen(30, 96)
        self.randLinksInnen.PositionSetzen(x + 2, y + 17)
        self.randLinksInnen.FarbeSetzen("cyan")
        self.eckeLUinnen = Kreis()
        self.eckeLUinnen.RadiusSetzen(15)
        self.eckeLUinnen.PositionSetzen(x + 17, y + 113)
        self.eckeLUinnen.FarbeSetzen("cyan")
        self.randUntenInnen = Rechteck()
        self.randUntenInnen.GroesseSetzen(56, 30)
        self.randUntenInnen.PositionSetzen(x + 17, y + 98)
        self.randUntenInnen.FarbeSetzen("cyan")
        self.randObenInnen = Rechteck()
        self.randObenInnen.GroesseSetzen(56, 30)
        self.randObenInnen.PositionSetzen(x + 17, y + 2)
        self.randObenInnen.FarbeSetzen("cyan")

        self.eckeROinnen = Kreis()
        self.eckeROinnen.RadiusSetzen(15)
        self.eckeROinnen.PositionSetzen(x+73, y + 17)
        self.eckeROinnen.FarbeSetzen("cyan")
        self.eckeRUinnen = Kreis()
        self.eckeRUinnen.RadiusSetzen(15)
        self.eckeRUinnen.PositionSetzen(x + 73, y + 113)
        self.eckeRUinnen.FarbeSetzen("cyan")
        self.randRechtsInnen = Rechteck()
        self.randRechtsInnen.GroesseSetzen(30, 96)
        self.randRechtsInnen.PositionSetzen(x + 58, y + 17)
        self.randRechtsInnen.FarbeSetzen("cyan")

        self.korpusInnen = Rechteck()
        self.korpusInnen.GroesseSetzen(60, 100)
        self.korpusInnen.PositionSetzen(x + 15, y + 15)
        self.korpusInnen.FarbeSetzen("cyan")

        self.griffLinks = Kreis()
        self.griffLinks.RadiusSetzen(5)
        self.griffLinks.PositionSetzen(x + 15, y + 45)
        self.griffLinks.FarbeSetzen("schwarz")

        self.griffMitte = Rechteck()
        self.griffMitte.GroesseSetzen(27, 10)
        self.griffMitte.PositionSetzen(x + 15, y + 40)
        self.griffMitte.FarbeSetzen("schwarz")

        self.griffRechts = Kreis()
        self.griffRechts.RadiusSetzen(5)
        self.griffRechts.PositionSetzen(x + 41, y + 45)
        self.griffRechts.FarbeSetzen("schwarz")

        self.ei1 = Kreis()
        self.ei1.RadiusSetzen(10)
        self.ei1.PositionSetzen(x + 21, y + 80)
        self.ei1.FarbeSetzen("orange")
        self.ei2 = Kreis()
        self.ei2.RadiusSetzen(10)
        self.ei2.PositionSetzen(x + 44, y + 80)
        self.ei2.FarbeSetzen("orange")
        self.ei3 = Kreis()
        self.ei3.RadiusSetzen(10)
        self.ei3.PositionSetzen(x + 67, y + 80)
        self.ei3.FarbeSetzen("orange")

        self.eierfach = Rechteck()
        self.eierfach.GroesseSetzen(70, 12)
        self.eierfach.PositionSetzen(x + 10, y + 85)
        self.eierfach.FarbeSetzen("grau")

    # Meldet den neuen Zustand an das Symbol
    def AenderungMelden(self, zustandNeu):
        if zustandNeu:
            self.ei1.FarbeSetzen("rot")
            self.ei2.FarbeSetzen("rot")
            self.ei3.FarbeSetzen("rot")
        else:
            self.ei1.FarbeSetzen("weiss")
            self.ei2.FarbeSetzen("weiss")
            self.ei3.FarbeSetzen("weiss")


class FensterSymbol:
    # Erzeugt das Fenstersymbol
    def __init__(self, x, y):
        hoehe = 100
        breite = hoehe

        self.links = Rechteck()
        self.links.GroesseSetzen(2, hoehe)
        self.links.PositionSetzen(x, y)
        self.links.FarbeSetzen("braun")

        self.oben = Rechteck()
        self.oben.GroesseSetzen(breite, 2)
        self.oben.PositionSetzen(x, y)
        self.oben.FarbeSetzen("braun")

        self.unten = Rechteck()
        self.unten.GroesseSetzen(breite, 2)
        self.unten.PositionSetzen(x, y + hoehe - 2)
        self.unten.FarbeSetzen("braun")

        self.rechts = Rechteck()
        self.rechts.GroesseSetzen(2, hoehe)
        self.rechts.PositionSetzen(x + breite - 2, y)
        self.rechts.FarbeSetzen("braun")

        self.kreuzSenkrecht = Rechteck()
        self.kreuzSenkrecht.GroesseSetzen(2, hoehe)
        self.kreuzSenkrecht.PositionSetzen(x + breite / 2, y)
        self.kreuzSenkrecht.FarbeSetzen("braun")

        self.kreuzWaagrecht = Rechteck()
        self.kreuzWaagrecht.GroesseSetzen(breite, 2)
        self.kreuzWaagrecht.PositionSetzen(x, y + hoehe / 2)
        self.kreuzWaagrecht.FarbeSetzen("braun")


class Hausmauer(Rechteck):
    def __init__(self, x=None, y=None):
        super().__init__()
        self.FarbeSetzen("grau")
        if x is not None and y is not None:
            self.PositionSetzen(x, y)


class HintergrundSymbol:
    # Legt die Wände, das Dach und die Beschriftung an
    def __init__(self):
        dachlinks = Dreieck()
        dachlinks.FarbeSetzen("rot")
        dachlinks.GroesseSetzen(550, 100)
        dachlinks.PositionSetzen(400, 0)
        unten = Hausmauer(150, 500)
        unten.GroesseSetzen(500, 5)
        mitteWaagrecht = Hausmauer(150, 300)
        mitteWaagrecht.GroesseSetzen(500, 5)
        oben = Hausmauer(150, 100)
        oben.GroesseSetzen(500, 5)
        links = Hausmauer(150, 100)
        links.GroesseSetzen(5, 400)
        rechts = Hausmauer(645, 100)
        rechts.GroesseSetzen(5, 400)
        mitteSenkrecht = Hausmauer(395, 100)
        mitteSenkrecht.GroesseSetzen(5, 400)
        FensterSymbol(250, 125)
        FensterSymbol(250, 325)
        FensterSymbol(500, 125)
        FensterSymbol(500, 325)

        wohnzimmer1 = Text()
        wohnzimmer1.TextSetzen("Wohnzimmer1")
        wohnzimmer1.FarbeSetzen("grau")
        wohnzimmer1.TextGroesseSetzen(15)
        wohnzimmer1.PositionSetzen(160, 120)

        wohnzimmer2 = Text()
        wohnzimmer2.TextSetzen("Wohnzimmer2")
        wohnzimmer2.FarbeSetzen("grau")
        wohnzimmer2.TextGroesseSetzen(15)
        wohnzimmer2.PositionSetzen(405, 120)

        kueche = Text()
        kueche.TextSetzen("Küche")
        kueche.FarbeSetzen("grau")
        kueche.TextGroesseSetzen(15)
        kueche.PositionSetzen(160, 320)

        esszimmer = Text()
        esszimmer.TextSetzen("Esszimmer1")
        esszimmer.FarbeSetzen("grau")
        esszimmer.TextGroesseSetzen(15)
        esszimmer.PositionSetzen(405, 320)


class Lampe(Hausgeraet):
    # Legt den Namen und das Darstellungssymbol fest
    def __init__(self, namensZusatz, bewegungsmelder, sonnensensor):
        super().__init__("Lampe" + namensZusatz)
        self.bewegung = False
        self.helligkeit = 0
        self.an = False
        self.automatik = True
        bewegungsmelder.BeobachterRegistrieren(self)
        sonnensensor.BeobachterRegistrieren(self)
        self.bewegungsmelder = bewegungsmelder
        self.sonnensensor = sonnensensor

    # Wird vom Sonnensensor bei einem neuen Helligkeitswert aufgerufen
    def WertaenderungMelden(self, wert, geraet):
        if geraet == self.sonnensensor:
            self.helligkeit = wert
            if self.automatik:
                self.an = self.bewegung and (self.helligkeit < 30)
                self.ZustandsAenderungMelden()

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        if self.an:
            return "ein"
        else:
            return "aus"

    # Schaltet die Komponente dauerhaft ein
    def Einschalten(self):
        super().Einschalten()
        self.an = True
        self.ZustandsAenderungMelden()

    # Schaltet die Komponente dauerhaft aus
    def Ausschalten(self):
        super().Ausschalten()
        self.an = False
        self.ZustandsAenderungMelden()

    # Schaltet die Komponente auf Automatikbetrieb
    def AutomatikAktivieren(self):
        super().AutomatikAktivieren()
        self.an = self.bewegung and (self.helligkeit < 30)
        self.ZustandsAenderungMelden()


class Rollladen:
    # Legt den Namen und das Darstellungssymbol fest
    def __init__(self, namensZusatz, symbol, windsensor, uhr):
        self.name = "Rolladen" + namensZusatz
        self.oben = False
        self.automatik = True
        windsensor.BeobachterRegistrieren(self)
        uhr.BeobachterRegistrieren(self)
        self.symbol = symbol
        self.windsensor = windsensor
        self.uhr = uhr
        self.windwert = 0
        self.uhrzeit = 0

    # Meldet den Namen des Geräts zurück
    # return Name
    def NameGeben(self):
        return self.name

    # Wird vom Windsensor bei einer neuen Windstärke aufgerufen
    # param wert die neue Windstärke
    def WertaenderungMeldenWindsensor(self, wert):
        self.windwert = wert
        if self.automatik:
            self.oben = (self.windwert > 6) or (self.uhrzeit > 60000) and (self.uhrzeit < 200000)
            self.symbol.AenderungMelden(self.oben)

    # Wird von der Uhr bei einer neuen Uhrzeit aufgerufen
    # param wert die neue Uhrzeit
    def WertänderungMeldenUhr(self, wert):
        self.uhrzeit = wert
        if self.automatik:
            self.oben = (self.windwert > 6) or (self.uhrzeit > 60000) and (self.uhrzeit < 200000)
            self.symbol.AenderungMelden(self.oben)

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        if self.oben:
            return "offen"
        else:
            return "geschlossen"

    # Schaltet die Komponente dauerhaft ein
    def Runterlassen(self):
        self.oben = False
        self.automatik = False
        self.symbol.AenderungMelden(self.oben)

    # Schaltet die Komponente dauerhaft aus
    def Hochziehen(self):
        self.oben = True
        self.automatik = False
        self.symbol.AenderungMelden(self.oben)

    # Schaltet die Komponente auf Automatikbetrieb
    def AutomatikAktivieren(self):
        self.oben = (self.windwert > 6) or (self.uhrzeit > 60000) and (self.uhrzeit < 200000)
        self.automatik = True
        self.symbol.AenderungMelden(self.oben)


class Heizkoerper(Hausgeraet):
    # Legt den Namen und das Darstellungssymbol fest
    def __init__(self, namensZusatz, thermometer):
        super().__init__("Heizkörper" + namensZusatz)
        self.temperatur = 0
        self.an = True
        self.automatik = True
        thermometer.BeobachterRegistrieren(self)

    # Wird vom Thermometer bei einer neuen Temperatur aufgerufen
    def WertaenderungMelden(self, wert, geraet):
        self.temperatur = wert
        if self.automatik:
            self.an = self.temperatur < 22
            self.ZustandsAenderungMelden()

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        if self.an:
            return "ein"
        else:
            return "aus"

    # Schaltet die Komponente dauerhaft ein
    def Einschalten(self):
        self.an = True
        self.automatik = False
        self.ZustandsAenderungMelden()

        # Schaltet die Komponente dauerhaft aus

    def Ausschalten(self):
        self.an = False
        self.automatik = False
        self.ZustandsAenderungMelden()

    # Schaltet die Komponente auf Automatikbetrieb
    def AutomatikAktivieren(self):
        self.an = self.temperatur < 22
        self.automatik = True
        self.ZustandsAenderungMelden()


class Kuehlschrank:
    # Legt den Namen und das Darstellungssymbol fest
    def __init__(self, namensZusatz, sy, waage):
        self.name = "Kühlschrank" + namensZusatz
        self.gewicht = 0
        self.leer = True
        self.symbol = sy
        self.symbol.AenderungMelden(self.leer)
        waage.BeobachterRegistrieren(self)

    # Meldet den Namen des Geräts zurück
    # return Name
    def NameGeben(self):
        return self.name

    # Wird von der Waage bei einem neuen Gewicht aufgerufen
    def WertaenderungMelden(self, wert):
        self.gewicht = wert
        self.leer = self.gewicht < 100
        self.symbol.AenderungMelden(self.leer)

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        if self.leer:
            return "Eierfach leer"
        else:
            return "Eierfach voll"

    # Tut nichts
    def Einschalten(self):
        pass

    # Tut nichts
    def Ausschalten(self):
        pass

    # Tut nichts
    def AutomatikAktivieren(self):
        pass


class Haus:
    def __init__(self):

        self.rolllaeden = []
        self.wohnzimmer1 = ZimmerVerwaltung("Wohnzimmer1")
        self.wohnzimmer2 = ZimmerVerwaltung("Wohnzimmer2")
        self.esszimmer = ZimmerVerwaltung("Esszimmer")
        self.kueche = ZimmerVerwaltung("Küche")
        self.haus = ZimmerVerwaltung("Haus")
        self.zimmer = []
        self.zimmer.append(self.haus)
        self.zimmer.append(self.wohnzimmer1)
        self.zimmer.append(self.wohnzimmer2)
        self.zimmer.append(self.esszimmer)
        self.zimmer.append(self.kueche)

        self.hintergrund = HintergrundSymbol()
        self.thermometerEsszimmer = Thermometer("Esszimmer")
        self.esszimmer.KomponenteEintragen(self.thermometerEsszimmer)
        self.heizkoerperSymbolEsszimmer = HeizkoerperSymbol(420, 440)
        self.heizkoerperEsszimmer = Heizkoerper("Esszimmer", self.thermometerEsszimmer)
        self.esszimmer.KomponenteEintragen(self.heizkoerperEsszimmer)
        self.heizkoerperEsszimmer.BeobachterRegistrieren(
            self.heizkoerperSymbolEsszimmer)

        self.uhrHaus = Uhr("Haus")
        self.haus.KomponenteEintragen(self.uhrHaus)
        self.windsensorHaus = Windsensor("Haus")
        self.haus.KomponenteEintragen(self.windsensorHaus)

        self.rollladenSymbolWohnzimmer1 = RollladenSymbol(250, 125)
        self.rollladenWohnzimmer1 = Rollladen(
            "Wohnzimmer1", self.rollladenSymbolWohnzimmer1, self.windsensorHaus, self.uhrHaus)
        self.rolllaeden.append(self.rollladenWohnzimmer1)

        self.wohnzimmer1.KomponenteEintragen(self.rollladenWohnzimmer1)
        self.rollladenSymbolWohnzimmer2 = RollladenSymbol(500, 125)
        self.rollladenWohnzimmer2 = Rollladen(
            "Wohnzimmer2", self.rollladenSymbolWohnzimmer2, self.windsensorHaus, self.uhrHaus)
        self.rolllaeden.append(self.rollladenWohnzimmer2)

        self.wohnzimmer2.KomponenteEintragen(self.rollladenWohnzimmer2)

        self.rollladenSymbolEsszimmer = RollladenSymbol(500, 325)
        self.rollladenEsszimmer = Rollladen(
            "Esszimmer", self.rollladenSymbolEsszimmer, self.windsensorHaus, self.uhrHaus)
        self.rolllaeden.append(self.rollladenEsszimmer)

        self.esszimmer.KomponenteEintragen(self.rollladenEsszimmer)
        self.rollladenSymbolKueche = RollladenSymbol(250, 325)
        self.rollladenKueche = Rollladen(
            "Küche", self.rollladenSymbolKueche, self.windsensorHaus, self.uhrHaus)
        self.rolllaeden.append(self.rollladenKueche)


        self.kueche.KomponenteEintragen(self.rollladenKueche)
        self.sonnensensorWohnzimmer2 = Sonnensensor("Wohnzimmer2")
        self.wohnzimmer2.KomponenteEintragen(self.sonnensensorWohnzimmer2)
        self.bewegungsmelderWohnzimmer2 = Bewegungsmelder("Wohnzimmer2")
        self.wohnzimmer2.KomponenteEintragen(self.bewegungsmelderWohnzimmer2)

        self.lampenSymbolWohnzimmer2_1 = LampenSymbol(450, 105)
        self.lampeWohnzimmer2_1 = Lampe(
            "Wohnzimmer2 1", self.bewegungsmelderWohnzimmer2, self.sonnensensorWohnzimmer2)
        self.wohnzimmer2.KomponenteEintragen(self.lampeWohnzimmer2_1)
        self.lampeWohnzimmer2_1.BeobachterRegistrieren(
            self.lampenSymbolWohnzimmer2_1)
        self.lampenSymbolWohnzimmer2_2 = LampenSymbol(550, 105)
        self.lampeWohnzimmer2_2 = Lampe(
            "Wohnzimmer2 2", self.bewegungsmelderWohnzimmer2, self.sonnensensorWohnzimmer2)
        self.wohnzimmer2.KomponenteEintragen(self.lampeWohnzimmer2_2)
        self.lampeWohnzimmer2_2.BeobachterRegistrieren(
            self.lampenSymbolWohnzimmer2_2)

        self.waageKueche = Waage("Küche")
        self.kueche.KomponenteEintragen(self.waageKueche)
        self.kuehlschrankSymbolKueche = KuehlschrankSymbol(158, 370)
        self.kuehlschrankKueche = Kuehlschrank("Küche", self.kuehlschrankSymbolKueche, self.waageKueche)
        self.kueche.KomponenteEintragen(self.kuehlschrankKueche)


     # Setzt das Thermometer im Esszimmer auf einen neuen Wert
     # param eingabe neuer Temperaturwert

    def ThermometerEsszimmerWertSetzen(self, eingabe):
        self.thermometerEsszimmer.WertSetzen(eingabe)

     # Setzt Uhr im Haus auf eine neue Zeit
     # param stunden neue Stundenangabe
    def UhrHausWertSetzen(self, stunden):
        self.uhrHaus.WertSetzen(stunden)

     # Setzt Windsensor im Haus auf einen neuen Wert
     # param eingabe neue Windstaerke
    def WindsensorHausWertSetzen(self, eingabe):
        self.windsensorHaus.WertSetzen(eingabe)

     # Setzt Bewegunsmelder in Wohnzimmer 2 auf einen neuen Wert
     # param an neuer Wert
    def BewegungsmelderWohnzimmer2WertSetzen(self, an):
        self.bewegungsmelderWohnzimmer2.WertSetzen(an)

     # Setzt Sonnensensor in Wohnzimmer 2 auf einen neuen Wert
     # param eingabe neue Helligkeit
    def SonnensensorWohnzimmer2WertSetzen(self, eingabe):
        self.sonnensensorWohnzimmer2.WertSetzen(eingabe)

     # Setzt Waage in der Kueche auf einen neuen Wert
     # param eingabe neues Gewicht
    def WaageKuecheWertSetzen(self, eingabe):
        self.waageKueche.WertSetzen(eingabe)

     # Schaltet den Heizkoerper im Esszimmer dauerhaft ein
    def HeizkoerperEsszimmerEinschalten(self):
        self.heizkoerperEsszimmer.Einschalten()

     # Schaltet den Heizkoerper im Esszimmer dauerhaft aus
    def HeizkoerperEsszimmerAusschalten(self):
        self.heizkoerperEsszimmer.Ausschalten()

     # Setzt den Heizkoerper im Esszimmer auf Automatikbetrieb
    def HeizkoerperEsszimmerAutomatikSetzen(self):
        self.heizkoerperEsszimmer.AutomatikAktivieren()

     # Schaltet die Lampe 1 in Wohnzimmer 2 dauerhaft ein
    def LampeWohnzimmer2_1Einschalten(self):
        self.lampeWohnzimmer2_1.Einschalten()

     # Schaltet die Lampe 1 in Wohnzimmer 2 dauerhaft aus
    def LampeWohnzimmer2_1Ausschalten(self):
        self.lampeWohnzimmer2_1.Ausschalten()

     # Setzt die Lampe 1 in Wohnzimmer 2 auf Automatikbetrieb
    def LampeWohnzimmer2_1AutomatikSetzen(self):
        self.lampeWohnzimmer2_1.AutomatikAktivieren()

     # Schaltet die Lampe 2 in Wohnzimmer 2 dauerhaft ein
    def LampeWohnzimmer2_2Einschalten(self):
        self.lampeWohnzimmer2_2.Einschalten()

     # Schaltet die Lampe 2 in Wohnzimmer 2 dauerhaft aus
    def LampeWohnzimmer2_2Ausschalten(self):
        self.lampeWohnzimmer2_2.Ausschalten()

     # Setzt die Lampe 2 in Wohnzimmer 2 auf Automatikbetrieb
    def LampeWohnzimmer2_2AutomatikSetzen(self):
        self.lampeWohnzimmer2_2.AutomatikAktivieren()

     # Zieht Rollladen in Wohnzimmer 1 dauerhaft hoch
    def RollladenWohnzimmer1Hochziehen(self):
        self.rollladenWohnzimmer1.Hochziehen()

     # Laesst Rollladen in Wohnzimmer 1 dauerhaft runter
    def RollladenWohnzimmer1Runterlassen(self):
        self.rollladenWohnzimmer1.Runterlassen()

    # Setzt Rollladen in Wohnzimmer 1 auf Automatikbetrieb
    def RollladenWohnzimmer1AutomatikSetzen(self):
        self.rollladenWohnzimmer1.AutomatikAktivieren()

    # Zieht Rollladen in Wohnzimmer 2 dauerhaft hoch
    def RollladenWohnzimmer2Hochziehen(self):
        self.rollladenWohnzimmer2.Hochziehen()

    # Lässt Rollladen in Wohnzimmer 2 dauerhaft runter
    def RollladenWohnzimmer2Runterlassen(self):
        self.rollladenWohnzimmer2.Runterlassen()

    # Setzt Rollladen in Wohnzimmer 2 auf Automatikbetrieb
    def RollladenWohnzimmer2AutomatikSetzen(self):
        self.rollladenWohnzimmer2.AutomatikAktivieren()

    # Zieht Rollladen im Esszimmer dauerhaft hoch
    def RollladenEsszimmerHochziehen(self):
        self.rollladenEsszimmer.Hochziehen()

    # Lässt Rollladen im Esszimmer dauerhaft runter
    def RollladenEsszimmerRunterlassen(self):
        self.rollladenEsszimmer.Runterlassen()

    # Setzt Rollladen im Esszimmer auf Automatikbetrieb
    def RollladenEsszimmerAutomatikSetzen(self):
        self.rollladenEsszimmer.AutomatikAktivieren()

    # Zieht Rollladen in der Küche dauerhaft hoch
    def RollladenKuecheHochziehen(self):
        self.rollladenKueche.Hochziehen()

    # Lässt Rollladen in der Küche dauerhaft runter
    def RollladenKuecheRunterlassen(self):
        self.rollladenKueche.Runterlassen()

    # Setzt Rollladen in der Küche auf Automatikbetrieb

    def RollladenKücheAutomatikSetzen(self):
        self.rollladenKueche.AutomatikAktivieren()

    # Schaltet alle Lampen dauerhaft ein
    def LampenEinschalten(self):
        self.wohnzimmer2.GeraeteEinschalten()

    # Schaltet alle Lampen dauerhaft aus
    def LampenAusschalten(self):
        self.wohnzimmer2.GeraeteAusschalten()

    # Schaltet alle Lampen auf Automatik
    def LampenAutomatikSetzen(self):
        self.wohnzimmer2.GeraeteAutomatikSetzen()

    # Zieht alle Rollläden dauerhaft hoch
    def RollladenHochziehen(self):
        for r in self.rolllaeden:
            r.Ausschalten()

    # Lässt alle Rollläden dauerhaft runter
    def RollladenRunterlassen(self):
        for r in self.rolllaeden:
            r.Einschalten()

    # Setzt alle Rollläden auf Automatikbetrieb
    def RollladenAutomatikSetzen(self):
        for r in self.rolllaeden:
            r.AutomatikAktivieren()

    # Zeigt die Zustände aller Geräte an
    def ZustaendeAnzeigen(self):
        for z in self.zimmer:
            z.ZustaendeAnzeigen()


h = Haus()
